#NoTrayIcon
#include <GUIConstants.au3>

; ---- Variables & Defaults ----
global $CurrentVersion = "1.3.3.12"
global $UpdateURL = "http://www.quate.net/archive/updates/portablescribus.txt"

; -- [App]
$DefaultFileName = "Scribus.exe"
$DefaultFileDir = "Scribus/"

; -- [Environment]
$DefaultUserProfile = "Profile/"
$DefaultGSPath = "Scribus/gs/"
$DefaultLanguage = "en"
$DefaultMultipleIstances = "false"
$DefaultTrayIcon = "true"
$DefaultTrayIconPath = "Scribus/scribus.ico"
$DefaultCleanRegistry = "true"
$DefaultModifyPrefs = "true"

; -- Script settings
$ScriptName = "scribus"
$ScriptIni = @ScriptDir & "\" & $ScriptName & ".ini"

; ---- Create ini if it doesn't already exist ----

If Not FileExists ($ScriptIni) Then
 IniWrite($ScriptIni, "App", "FileName", $DefaultFileName)
 IniWrite($ScriptIni, "App", "FileDir", $DefaultFileDir)
 IniWrite($ScriptIni, "Environment", "UserProfile", $DefaultUserProfile)
 IniWrite($ScriptIni, "Environment", "GSPath", $DefaultGSPath)
 IniWrite($ScriptIni, "Environment", "Language", $DefaultLanguage)
 IniWrite($ScriptIni, "Environment", "MultipleIstances", $DefaultMultipleIstances)
 IniWrite($ScriptIni, "Environment", "TrayIcon", $DefaultTrayIcon)
 IniWrite($ScriptIni, "Environment", "TrayIconPath", $DefaultTrayIconPath)
 IniWrite($ScriptIni, "Environment", "ModifyPrefs", $DefaultModifyPrefs)
 IniWrite($ScriptIni, "Windows", "CleanRegistry", $DefaultCleanRegistry)
EndIf

; ---- Get configurations from ini file ----

$FileName = IniRead($ScriptIni, "App", "FileName", $DefaultFileName)
$FileDir = IniRead($ScriptIni, "App", "FileDir", $DefaultFileDir)
$UserProfile = IniRead($ScriptIni, "Environment", "UserProfile", $DefaultUserProfile)
$GSPath = IniRead($ScriptIni, "Environment", "GSPath", $DefaultGSPath)
$Language = IniRead($ScriptIni, "Environment", "Language", $DefaultLanguage)
$MultipleIstances = IniRead($ScriptIni, "Environment", "MultipleIstances", $DefaultMultipleIstances)
$TrayIcon = IniRead($ScriptIni, "Environment", "TrayIcon", $DefaultTrayIcon)
$TrayIconPath = IniRead($ScriptIni, "Environment", "TrayIconPath", $DefaultTrayIconPath)
$ModifyPrefs = IniRead($ScriptIni, "Environment", "ModifyPrefs", $DefaultModifyPrefs)
$CleanRegistry = IniRead($ScriptIni, "Windows", "CleanRegistry", $DefaultCleanRegistry)

; ---- Make sure user config dir exists ----
If Not FileExists($UserProfile) Then
 DirCreate($UserProfile)
EndIf

; ---- Declare settings ----

;EnvSet('USERPROFILE', @ScriptDir & "/" & $UserProfile)
EnvSet('HOME', @ScriptDir & "/" & $UserProfile)
EnvSet('PATH', $GSPath)
EnvSet('LANG', $Language)

; ---- Already running? ----
If $MultipleIstances = "false" Then
 If ProcessExists($FileName) Then
  MsgBox(0, "Portable Scribus", "Scribus is already open.")
  Exit
 EndIf
EndIf

; ---- Replace Preferences accordingly if necessary. ----
If $ModifyPrefs = "true" Then
	; The Scribus preference file contains data about where Scribus documents are located. This would be a problem if a user used two different computers and saved docs in the profile area or whatnot.
	$prefsfile = @ScriptDir & "\" & $UserProfile & "\.scribus\prefs13.xml"
	$file = FileOpen($prefsfile, 0)
	
	If $file = -1 Then
	Else
		$fileread = FileRead($file)
		$lineread = FileReadLine($file, 76)
		FileClose($file)

		$string1 = stringbetween($lineread, '<attribute key="pdf" value="', '"/>')
		$newline = StringReplace($lineread, $string1, @ScriptDir & '\Profile')
		$newfile = StringReplace($fileread, $lineread, $newline)
		FileDelete($prefsfile)
		FileWrite($prefsfile, $newfile)
	EndIf
EndIf

; ---- Check if program exists ----
If Not FileExists(@ScriptDir & "\" & $FileDir) Then
 MsgBox(0, "Portable Scribus", "Could not find Scribus executable.")
 Exit
EndIf

; First create tray icon!
; ---- Tray Icon ----
If $TrayIcon = "true" Then
 Opt("TrayMenuMode", 1)
 $AboutTray = TrayCreateItem("About")
 $UpdateTray = TrayCreateItem("Check for Updates")
 $SeparatorTray = TrayCreateItem("")
 $ExitTray = TrayCreateItem("Exit")
 TraySetState()
 TraySetIcon($TrayIconPath, "")
 TraySetToolTip("Portable Scribus " & $CurrentVersion)

Run(@ScriptDir & "\" & $FileDir & "" & $FileName, "")

 While 1
  If NOT ProcessExists($FileName) Then
   ; Delete Qt Plugins registry key. Question is, will this ruin programs that need this key, or will they recreate this qt registry key, as Scribus does.
   ;RegDelete("HKEY_LOCAL_MACHINE\SOFTWARE\Trolltech\Qt Plugins 3.3\")
   If $CleanRegistry = "true" Then
    RegDelete("HKEY_LOCAL_MACHINE\SOFTWARE\Trolltech\")
   EndIf
   ; Exit program.
   ExitLoop
   Exit
  EndIf
  $msg = TrayGetMsg()
  If $msg = $ExitTray Then
   If ProcessExists($FileName) Then
    Dim $iMsgBoxAnswer
    $iMsgBoxAnswer = MsgBox(292,"Close Portable Scribus?", "Are you sure you want to close Portable Scribus?" & @CRLF & "" & @CRLF & "Make sure you have saved your work before closing Portable Scribus.")
    Select
     Case $iMsgBoxAnswer = 6 ;Yes
      ; Exit
      ProcessClose($FileName)
      ExitLoop
      Exit
     Case $iMsgBoxAnswer = 7 ;No
     ; Do nothing
    EndSelect
   EndIf
  ElseIf $msg = $UpdateTray Then
   Update()
  ElseIf $msg = $AboutTray Then
   ;MsgBox(0, "About Portable Scribus", "Portable Scribus " & $CurrentVersion & " created by Quate.net (www.quate.net)." & @CRLF & "The official Scribus was created by the Scribus team at www.scribus.net." & @CRLF & "" & @CRLF & "For more information about Scribus, see the help menu.")
   About()
  EndIf
  Sleep(10)
 WEnd
Else
	; Run program with no tray icon.
	Run(@ScriptDir & "\" & $FileDir & "" & $FileName, "")
	While 1
		If NOT ProcessExists($FileName) Then
			; Delete Qt Plugins registry key. Question is, will this ruin programs that need this key, or will they recreate this qt registry key, as Scribus does.
			;RegDelete("HKEY_LOCAL_MACHINE\SOFTWARE\Trolltech\Qt Plugins 3.3\")
			If $CleanRegistry = "true" Then
				RegDelete("HKEY_LOCAL_MACHINE\SOFTWARE\Trolltech\")
			EndIf
			; Exit program.
			ExitLoop
			Exit
		EndIf
		Sleep(2000)
	WEnd
EndIf

; ---- Update Function ----
Func Update()

$Find = '<'
$Before = '>'
$list_url = $UpdateURL

; Empty variables (temp)
$Version = ""
$Link = ""

If @OSTYPE = "WIN32_NT" Then
	TrayTip("Checking for updates", "Checking for Portable Scribus updates.", 1)
	Sleep("1500")
	TrayTip("", "", 1)
Else
	MsgBox(0, "Checking", "Checking for updates...", 1)
EndIf

$save_locate = @TempDir & "/foo.html"
FileDelete($save_locate)
$body = InetGet($list_url, $save_locate, 1)
$sfile = FileOpen($save_locate, 0)
$num = 0
$while_found = 0
While $while_found < 2
    $num = $num + 1
    $sline = FileReadLine($sfile, $num)
    If @error Then
        MsgBox(262208, "Failed", "Error in downloading update information.", 5)
	    FileClose($save_locate)
        ExitLoop
    EndIf
    If StringInStr($sline, $Find) Then
        If $Before NOT = "" Then
        $Found = stringbetween($sline, $Find, $Before)
        ;MsgBox(64, "Found", "The string " & $Found & " was found on line " & $num & ".", 5)
		$while_found = $while_found + 1
		If ($while_found = 1) Then
			$Version = $Found
		ElseIf ($while_found = 2) Then
			$Link = $Found
		EndIf
		
        EndIf
    EndIf
WEnd
FileDelete($save_locate)

If ($CurrentVersion < $Version) Then
 MsgBox(64, "New Version Found", "Portable Scribus version " & $Version & " was found, and can be downloaded at: " & @CRLF & "" & $Link)
Else
 MsgBox(64, "No Updates Found", "No update for Portable Scribus was found.")
EndIf

EndFunc

; From: http://www.autoitscript.com/forum/index.php?s=&showtopic=35090&view=findpost&p=257028
Func stringbetween($str, $start, $end)
    $pos = StringInStr($str, $start)
    If Not @error Then
        $str = StringTrimLeft($str, $pos + StringLen($start) - 1)
        $pos = StringInStr($str, $end)
        If Not @error Then
            $str = StringTrimRight($str, StringLen($str) - $pos + 1)
            Return $str
        EndIf
    EndIf
EndFunc

; ALTERNATIVE 1 to the "stringbetween" function
;Func _SRE_BetweenEX($s_String, $s_Start, $s_End, $iCase = 'i')
;    If $iCase <> 'i' Then $iCase = ''
;    $a_Array = StringRegExp ($s_String, '(?' & $iCase & _
;            ':' & $s_Start & ')(.*?)(?' & $iCase & _
;            ':' & $s_End & ')', 3)
;    If @extended & IsArray($a_Array) Then Return $a_Array
;    Return SetError(1, 0, 0)
;EndFunc   ;==>_SRE_BetweenEX

; ALTERNATIVE 2 to the "stringbetween" function
;Func _StringBetween($s_String, $s_Start, $s_End = 0)
;    $s_Start = StringInStr($s_String, $s_Start)+StringLen($s_Start)
;    return StringMid($s_String, $s_Start, StringInStr($s_String, $s_End)-$s_Start)
;EndFunc

; ---- About Function ----
Func About()
	$gui_about = GUICreate("About Portable Scribus", 300, 150)
	GuiSetIcon($TrayIconPath, 0)
	$Label = GuiCtrlCreateLabel("Portable Scribus " & $CurrentVersion & "" & @CRLF & "" & @CRLF & "Program created by Quate (www.quate.net), licensed under the GPL." & @CRLF & "The official Scribus was created by the Scribus team at www.scribus.net." & @CRLF & "" & @CRLF & "For more information about Scribus, see the help menu.", 10, 10, 280, 115)
	$OkayButton  = GUICtrlCreateButton("OK", 120, 120, 72, 22)
	GUICtrlSetState(-1, $GUI_FOCUS)
	GUISetState()
	
	While 1
		$msg = GUIGetMsg()
		If $msg = $GUI_EVENT_CLOSE Then
			GUIDelete()
			ExitLoop
		ElseIf $msg = $OkayButton Then
			GUIDelete()
			ExitLoop
		EndIf
	WEnd
EndFunc